The OXFuse library is free software.
This library is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

**Official repositories**:

- [https://gitlab.com/os-18/oxfuse](https://gitlab.com/os-18/oxfuse)
- [https://codeberg.org/os-18/oxfuse](https://codeberg.org/os-18/oxfuse)

---

## OXFuse

OXFuse (Object-oriented eXtension of FUSE) is a binding to [libfuse3](https://github.com/libfuse/libfuse)
for the D programming language.

### Features

- To create your own file system, you should inherit from the `oxfuse.FileSystem` class and overload any methods (file operations).

- This library allows to use fourty file system operations from libfuse (all except `write_buf()`/`read_buf()`).

- The behavior of all non-overloaded functions is determined by FUSE and the OS kernel.

- Mounting features require the mount point to be set first using the `setMountPoint()` method of the FileSystem class.

- The library doesn't contain bindings to functions that parse command line arguments. Parsing is proposed to be done using the D language tools.

- Error situations with accompanying messages and `errno` are based on `FuseException` and the `fe()` function.

### See also

The "examples" directory contains several different implementations of file systems.

Also see official wiki of __libfuse__: [https://github.com/libfuse/libfuse/wiki](https://github.com/libfuse/libfuse/wiki).

---

## Build from source

### Preparing

You must have [make](https://www.gnu.org/software/make/)
and at least one of these compilers:

- [Digital Mars D Compiler](https://dlang.org/download.html#dmd)
- [LLVM D Compiler](https://wiki.dlang.org/LDC)
- [GNU D Compiler](https://www.gdcproject.org)

Also the library needs **libfuse3**.

### Simple building and installing

Creating of static and dynamic libraries for the **ldc2** compiler (by default):

```
make
```

Installation (root rights may be required):

```
make install
```

After that, the library for ldc2 is ready for use.

You can install the library in any other directory:

```
make install DESTDIR=/home/$USER/sandbox
```

Uninstalling:

```
make uninstall
```

If you installed in an alternative directory:

```
make uninstall DESTDIR=/home/$USER/sandbox
```

### Specific building

Compiler by default is **ldc2**.
You can choose other compiler (for example, **dmd**):

```
make DC=dmd
```

Installing:

```
make install DC=dmd
```

Uninstalling:

```
make uninstall DC=dmd
```

---


## Feedback

Questions, suggestions, comments, bugs:

**tech.vindex@gmail.com**

Also use the repository service tools.

---
